import 'package:bmi_global/utils/user.dart';
import 'package:flutter/material.dart';
import 'package:bmi_global/layout/gateway.dart';
import 'package:bmi_global/layout/route.dart';
import 'package:bmi_global/provider/auth.dart';
import 'package:bmi_global/provider/config.dart';
import 'package:bmi_global/provider/app_info.dart';
import 'package:bmi_global/utils/proceed.dart';
import 'package:provider/provider.dart';

class BmiApp<U extends AuthUser, R extends AppRoute> extends StatelessWidget {
  const BmiApp({
    Key? key,
    required this.authProvider,
    required this.appInfoProvider,
    required this.configProvider,
  }) : super(key: key);
  final AuthProvider<U> authProvider;
  final AppInfoProvider appInfoProvider;
  final ConfigProvider configProvider;
  @override
  Widget build(BuildContext context) {
    final defaultRoute = configProvider.defaultRoute;
    Widget getPage(BuildContext context, AppRoute appRoute, {argument}) {
      return appRoute.builder(context, argument: argument) ??
          configProvider.build404Page(context);
    }

    return MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => configProvider),
        ChangeNotifierProvider(create: (_) => appInfoProvider),
        ChangeNotifierProvider(create: (_) => authProvider),
        Provider(create: (_) => RouteSettings(name: defaultRoute.route)),
      ],
      child: MaterialApp(
        title: 'B2B',
        themeMode: configProvider.themeMode,
        theme: ThemeData.light(),
        darkTheme: ThemeData.dark(),
        initialRoute: configProvider.defaultRoute.route,
        home: Builder(builder: (context) {
          return getPage(context, defaultRoute);
        }),
        onUnknownRoute: (RouteSettings settings) {
          return PageRouteBuilder(
            pageBuilder: (context, _, __) {
              return Provider(
                create: (_) => settings,
                child: configProvider.build404Page(context),
              );
            },
            transitionDuration: const Duration(milliseconds: 200),
            reverseTransitionDuration: const Duration(milliseconds: 100),
            transitionsBuilder: (_, a, __, c) {
              return FadeTransition(opacity: a, child: c);
            },
          );
        },
        onGenerateRoute: (RouteSettings settings) {
          final route = settings.name;
          final appRoute = appRouteFromRoute(route);
          if (appRoute == null) return null;
          if (appRoute.isMainPage) {
            return PageRouteBuilder(
              pageBuilder: (context, _, __) {
                return WillPopScope(
                  onWillPop: () async {
                    if (Navigator.canPop(context)) return true;
                    return await proceed(
                      context,
                      "Exit App",
                      "Sure, you want to Exit?",
                    );
                  },
                  child: Provider(
                    create: (context) => settings,
                    child: getPage(context, appRoute,
                        argument: settings.arguments),
                  ),
                );
              },
              transitionDuration: const Duration(milliseconds: 250),
              reverseTransitionDuration: const Duration(milliseconds: 150),
              transitionsBuilder: (_, a, __, c) {
                return FadeTransition(opacity: a, child: c);
              },
            );
          } else {
            return MaterialPageRoute(
              builder: (context) {
                return Provider(
                  create: (_) => settings,
                  child:
                      getPage(context, appRoute, argument: settings.arguments),
                );
              },
            );
          }
        },
        builder: GateWayPage<U>.build,
      ),
    );
  }
}
